@@ -38,28 +38,40 @@
 #include "models/DiscModel.h"
 #include "models/DiscDelegate.h"
 #include "wiitdb/Covers.h"
-#include "datacache/DataNetworkCache.h"
 #include "ProgressDialog.h"
 #include "PropertiesDialog.h"
 #include "Properties.h"
 #include "main.h"
-#include "donation/PaypalDonationWidget.h"
-#include "UpdateChecker/pUpdateChecker.h"
 
-#include <pTranslationManager.h>
-#include <pTranslationDialog.h>
+#include <FreshCore/pNetworkAccessManager>
+#include <FreshCore/pTranslationManager>
+#include <FreshGui/pTranslationDialog>
+#include <FreshGui/pUpdateChecker>
+#include <FreshGui/pPaypalButton>
+#include "models/pPartitionModel.h" // will be part of fresh library in a next version
 
+#include <QMenuBar>
+#include <QMenu>
 #include <QFileSystemModel>
 #include <QFileDialog>
 #include <QProcess>
 #include <QMessageBox>
-#include <QSpacerItem>
+#include <QInputDialog>
+#include <QPainter>
+#include <QNetworkReply>
+#include <QDesktopServices>
 #include <QDebug>
 
+#define COVER_DISC_SIZE QSize( 160, 160 )
+#define COVER_SIZE QSize( 160, 224 )
+
 UIMain::UIMain( QWidget* parent )
 	: QMainWindow( parent )
 {
-	mCache = new DataNetworkCache( this );
+#if defined( Q_OS_MAC )
+	mMenuBar = new QMenuBar;
+#endif
+	mCache = pNetworkAccessManager::instance();
 	mUpdateChecker = new pUpdateChecker( this );
 	mUpdateChecker->setDownloadsFeedUrl( QUrl( APPLICATION_DOWNLOADS_FEED ) );
 	mUpdateChecker->setVersion( APPLICATION_VERSION );
@@ -69,20 +81,34 @@ UIMain::UIMain( QWidget* parent )
 	setWindowTitle( QString( "%1 v%2" ).arg( APPLICATION_NAME ).arg( APPLICATION_VERSION_STR ) );
 	setUnifiedTitleAndToolBarOnMac( true );
 	setupUi( this );
+
+#if defined( Q_OS_MAC )
+	mMenuBar->addMenu( "Fake" )->addAction( aProperties );
+#endif
 	
 	centralVerticalLayout->setMenuBar( qmtbInfos );
+	qmtbInfos->layout()->setMargin( 5 );
+	qmtbInfos->queuedMessageWidget()->setContentsMargins( 5, 0, 5, 0 );
 	qmtbInfos->setVisible( false );
 	
 	dwTools->toggleViewAction()->setIcon( QIcon( ":/icons/256/tools.png" ) );
 	dwCovers->toggleViewAction()->setIcon( QIcon( ":/icons/256/covers.png" ) );
 	
-	mDonationWidget = new PaypalDonationWidget( this );
+	mDonationWidget = new pPaypalButton( this );
 	mDonationWidget->setBusinessId( "5R924WYXJ6BAW" );
 	mDonationWidget->setItemName( "QWBFS Manager" );
 	mDonationWidget->setItemId( "QWBFS-DONATION" );
 	mDonationWidget->setCurrencyCode( "EUR" );
 	
+	mActions = new QMenu( this );
+	mActions->setIcon( aConvertToWBFSFiles->icon() );
+	mActions->addAction( aConvertToWBFSFiles );
+	mActions->addAction( aConvertToISOFiles );
+	mActions->addAction( aRenameDiscsInFolder );
+	
 	toolBar->insertAction( aAbout, mUpdateChecker->menuAction() );
+	toolBar->addAction( mActions->menuAction() );
+	toolBar->addSeparator();
 	toolBar->addAction( dwTools->toggleViewAction() );
 	toolBar->addAction( dwCovers->toggleViewAction() );
 	QWidget* spacerWidget = new QWidget( toolBar );
@@ -103,36 +129,56 @@ UIMain::UIMain( QWidget* parent )
 	
 	lvFiles->setModel( mFilesModel );
 	
-	mExportModel = new QWBFS::Model::DiscModel( this );
-	
-	lvExport->setModel( mExportModel );
-	lvExport->setItemDelegate( new QWBFS::Model::DiscDelegate( mExportModel, mCache ) );
+	lvExport->initialize( 0, mCache );
 	
 	mLastDiscId = -1;
 	
 	pwMainView->setMainView( true );
 	pwMainView->showHideImportViewButton()->setChecked( false );
 	connectView( pwMainView );
-	tbReloadDrives->click();
-	aReloadPartitions->trigger();
+	
+	qmtbInfos->installEventFilter( this );
+	lWiiTDB->installEventFilter( this );
 	
 	localeChanged();
 	
-	connect( mCache, SIGNAL( dataCached( const QUrl& ) ), this, SLOT( dataNetworkCache_dataCached( const QUrl& ) ) );
-	connect( mCache, SIGNAL( error( const QString&, const QUrl& ) ), this, SLOT( dataNetworkCache_error( const QString&, const QUrl& ) ) );
-	connect( mCache, SIGNAL( invalidated() ), this, SLOT( dataNetworkCache_invalidated() ) );
+	connect( mCache, SIGNAL( finished( QNetworkReply* ) ), this, SLOT( networkAccessManager_finished( QNetworkReply* ) ) );
+	connect( mCache, SIGNAL( cached( const QUrl& ) ), this, SLOT( networkAccessManager_cached( const QUrl& ) ) );
+	connect( mCache, SIGNAL( error( const QUrl&, const QString& ) ), this, SLOT( networkAccessManager_error( const QUrl&, const QString& ) ) );
+	connect( mCache, SIGNAL( cacheCleared() ), this, SLOT( networkAccessManager_cacheCleared() ) );
 }
 
 UIMain::~UIMain()
 {
+#if defined( Q_OS_MAC )
+	delete mMenuBar;
+#endif
 	//qWarning() << Q_FUNC_INFO;
 }
 
-DataNetworkCache* UIMain::cache() const
+pNetworkAccessManager* UIMain::cache() const
 {
 	return mCache;
 }
 
+pQueuedMessageToolBar* UIMain::messageToolBar() const
+{
+	return qmtbInfos;
+}
+
+bool UIMain::event( QEvent* event )
+{
+	switch ( event->type() ) {
+		case QEvent::LocaleChange:
+			localeChanged();
+			break;
+		default:
+			break;
+	}
+	
+	return QMainWindow::event( event );
+}
+
 void UIMain::showEvent( QShowEvent* event )
 {
 	QMainWindow::showEvent( event );
@@ -156,31 +202,58 @@ void UIMain::closeEvent( QCloseEvent* event )
 	QMainWindow::closeEvent( event );
 }
 
-bool UIMain::event( QEvent* event )
+bool UIMain::eventFilter( QObject* object, QEvent* event )
 {
-	switch ( event->type() ) {
-		case QEvent::LocaleChange:
-			localeChanged();
-			break;
-		default:
-			break;
+	if ( object == qmtbInfos ) {
+		if ( event->type() == QEvent::Paint ) {
+			if ( qmtbInfos->queuedMessageWidget()->pendingMessageCount() > 0 ) {
+				QPainter painter( qmtbInfos );
+				QBrush brush;
+				
+				qmtbInfos->queuedMessageWidget()->currentMessageInformations( 0, &brush, 0 );
+				
+				painter.setRenderHint( QPainter::Antialiasing );
+				painter.setPen( QPen( brush.color().darker( 150 ), 0.5 ) );
+				painter.setBrush( brush );
+				painter.drawRoundedRect( qmtbInfos->rect().adjusted( 10 -4, -9, -10 +4, -1 ), 9, 9 );
+				
+				return true;
+			}
+		}
+	}
+	else if ( object == lWiiTDB ) {
+		if ( event->type() == QEvent::MouseButtonPress ) {
+			QDesktopServices::openUrl( lWiiTDB->toolTip() );
+		}
 	}
 	
-	return QMainWindow::event( event );
+	return QMainWindow::eventFilter( object, event );
+}
+
+void UIMain::connectView( PartitionWidget* widget )
+{
+	connect( widget, SIGNAL( openViewRequested() ), this, SLOT( openViewRequested() ) );
+	connect( widget, SIGNAL( closeViewRequested() ), this, SLOT( closeViewRequested() ) );
+	connect( widget, SIGNAL( coverRequested( const QString& ) ), this, SLOT( coverRequested( const QString& ) ) );
 }
 
 void UIMain::localeChanged()
 {
 	retranslateUi( this );
+	mActions->setTitle( tr( "Actions" ) );
 }
 
-void UIMain::loadProperties()
+void UIMain::loadProperties( bool firstInit )
 {
 	Properties properties( this );
 	
-	mCache->setDiskCacheSize( properties.cacheDiskSize() );
-	mCache->setMemoryCacheSize( properties.cacheMemorySize() );
-	mCache->setWorkingPath( properties.cacheUseTemporaryPath() ? properties.temporaryPath() : properties.cacheWorkingPath() );
+	PartitionComboBox::partitionModel()->addPartitions( properties.customPartitions() );
+	
+	tbReloadDrives->click();
+	aReloadPartitions->trigger();
+	
+	mCache->setMaximumCacheSize( properties.cacheDiskSize() );
+	mCache->setCacheDirectory( properties.cacheUseTemporaryPath() ? properties.temporaryPath() : properties.cacheWorkingPath() );
 	
 	QNetworkProxy proxy( properties.proxyType() );
 	proxy.setHostName( properties.proxyServer() );
@@ -190,12 +263,18 @@ void UIMain::loadProperties()
 	
 	QNetworkProxy::setApplicationProxy( proxy );
 	
-	mDonationWidget->cache()->setDiskCacheSize( mCache->diskCacheSize() );
-	mDonationWidget->cache()->setMemoryCacheSize( mCache->memoryCacheSize() );
-	mDonationWidget->cache()->setWorkingPath( mCache->workingPath() );
-	
-	mUpdateChecker->setLastUpdated( properties.updateLastUpdated() );
-	mUpdateChecker->setLastChecked( properties.updateLastChecked() );
+	if ( firstInit ) {
+		mUpdateChecker->setLastUpdated( properties.updateLastUpdated() );
+		mUpdateChecker->setLastChecked( properties.updateLastChecked() );
+		
+		const QModelIndex index = mFoldersModel->index( properties.selectedPath() );
+		
+		properties.restoreState( this );
+		tvFolders->setCurrentIndex( index );
+		tvFolders->scrollTo( index );
+		on_tvFolders_activated( index );
+		pwMainView->setCurrentPartition( properties.selectedPartition() );
+	}
 	
 	pTranslationManager* translationManager = pTranslationManager::instance();
 	translationManager->setTranslationsPaths( properties.translationsPaths() );
@@ -211,32 +290,32 @@ void UIMain::loadProperties()
 		widget->setLocale( translationManager->currentLocale() );
 	}
 	
-	properties.restoreState( this );
+	foreach ( ListView* view, findChildren<ListView*>() ) {
+		view->setViewMode( properties.viewMode() );
+		view->setViewIconType( properties.viewIconType() );
+	}
 }
 
 void UIMain::saveProperties()
 {
+	const QModelIndex index = tvFolders->selectionModel()->selectedIndexes().value( 0 );
+	const QString selectedPath = mFoldersModel->filePath( index );
 	Properties properties;
 	
 	properties.setUpdateLastUpdated( mUpdateChecker->lastUpdated() );
 	properties.setUpdateLastChecked( mUpdateChecker->lastChecked() );
+	properties.setSelectedPath( selectedPath );
+	properties.setSelectedPartition( pwMainView->currentPartition() );
+	properties.setCustomPartitions( PartitionComboBox::partitionModel()->customPartitions() );
 	properties.saveState( this );
 }
 
-void UIMain::connectView( PartitionWidget* widget )
-{
-	connect( widget, SIGNAL( openViewRequested() ), this, SLOT( openViewRequested() ) );
-	connect( widget, SIGNAL( closeViewRequested() ), this, SLOT( closeViewRequested() ) );
-	connect( widget, SIGNAL( coverRequested( const QString& ) ), this, SLOT( coverRequested( const QString& ) ) );
-}
-
 void UIMain::changeLocaleRequested()
 {
 	pTranslationManager* translationManager = pTranslationManager::instance();
-	const QString locale = pTranslationDialog::getLocale( translationManager );
+	const QString locale = pTranslationDialog::getLocale( translationManager, this );
 	
-	if ( !locale.isEmpty() )
-	{
+	if ( !locale.isEmpty() ) {
 		Properties properties;
 		properties.setTranslationsPaths( translationManager->translationsPaths() );
 		properties.setLocaleAccepted( true );
@@ -248,14 +327,13 @@ void UIMain::changeLocaleRequested()
 
 void UIMain::propertiesChanged()
 {
-	loadProperties();
+	loadProperties( false );
 }
 
 void UIMain::openViewRequested()
 {
 	PartitionWidget* pw = new PartitionWidget( this );
 	pw->setMainView( false );
-	pw->setPartitions( mPartitions );
 	pw->showHideImportViewButton()->setChecked( false );
 	connectView( pw );
 	sViews->addWidget( pw );
@@ -270,152 +348,110 @@ void UIMain::coverRequested( const QString& id )
 {
 	mLastDiscId = id;
 	
-	const QUrl urlCD = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::Disc, id );
-	const QUrl urlCDCustom = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::DiscCustom, id );
-	const QUrl urlCover = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::Cover, id );
-	
-	lCDCover->clear();
-	lCover->clear();
-	
-	if ( mCache->hasCachedData( urlCD ) || mCache->hasCachedData( urlCDCustom ) || mCache->hasCachedData( urlCover ) ) {
-		dataNetworkCache_dataCached( QUrl() );
-	}
-	
-	if ( !lCDCover->pixmap() ) {
-		mCache->cacheData( urlCD );
-	}
-	
-	if ( !lCover->pixmap() ) {
-		mCache->cacheData( urlCover );
-	}
+	networkAccessManager_cached( QUrl( WIITDB_DOMAIN ) );
+	QWBFS::WiiTDB::coverDiscPixmap( id, mCache, COVER_DISC_SIZE );
+	QWBFS::WiiTDB::coverBoxPixmap( id, mCache, COVER_SIZE );
 }
 
 void UIMain::progress_jobFinished( const QWBFS::Model::Disc& disc )
 {
-	mExportModel->updateDisc( disc );
+	lvExport->model()->updateDisc( disc );
 }
 
-void UIMain::dataNetworkCache_dataCached( const QUrl& url )
+void UIMain::networkAccessManager_finished( QNetworkReply* reply )
 {
-	Q_UNUSED( url );
-	
-	// update all views
-	const QList<QAbstractItemView*> views = findChildren<QAbstractItemView*>();
-	
-	foreach ( QAbstractItemView* view, views ) {
-		view->viewport()->update();
-	}
-	
-	// update preview dock
-	if ( mLastDiscId.isEmpty() ) {
+	reply->deleteLater();
+}
+
+void UIMain::networkAccessManager_cached( const QUrl& url )
+{
+	if ( !url.toString().startsWith( WIITDB_DOMAIN, Qt::CaseInsensitive ) && url != QUrl() ) {
 		return;
 	}
 	
-	const QUrl urlCD = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::Disc, mLastDiscId );
-	const QUrl urlCDCustom = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::DiscCustom, mLastDiscId );
-	const QUrl urlCover = QWBFS::WiiTDB::Covers::url( QWBFS::WiiTDB::Covers::Cover, mLastDiscId );
+	// update all models
+	const QString id = QFileInfo( url.path() ).baseName();
+	const QList<QWBFS::Model::DiscModel*> models = findChildren<QWBFS::Model::DiscModel*>();
 	
-	if ( mCache->hasCachedData( urlCD ) ) {
-		lCDCover->setPixmap( mCache->cachedPixmap( urlCD ) );
+	foreach ( QWBFS::Model::DiscModel* model, models ) {
+		const QModelIndex index = model->index( id );
+		
+		model->setData( index, QVariant(), Qt::DecorationRole );
 	}
 	
-	if ( mCache->hasCachedData( urlCDCustom ) ) {
-		lCDCover->setPixmap( mCache->cachedPixmap( urlCDCustom ) );
-	}
-	
-	if ( mCache->hasCachedData( urlCover ) ) {
-		lCover->setPixmap( mCache->cachedPixmap( urlCover ) );
-	}
+	lCDCover->setPixmap( QWBFS::WiiTDB::coverDiscPixmap( mLastDiscId, mCache, COVER_DISC_SIZE ) );
+	lCover->setPixmap( QWBFS::WiiTDB::coverBoxPixmap( mLastDiscId, mCache, COVER_SIZE ) );
 }
 
-void UIMain::dataNetworkCache_error( const QString& message, const QUrl& url )
+void UIMain::networkAccessManager_error( const QUrl& url, const QString& message )
 {
-	switch ( QWBFS::WiiTDB::Covers::type( url ) )
+	if ( !url.toString().startsWith( WIITDB_DOMAIN, Qt::CaseInsensitive ) ) {
+		return;
+	}
+	
+	const QString id = QFileInfo( url.path() ).baseName();
+	
+	switch ( QWBFS::WiiTDB::urlCover( url ) )
 	{
-		case QWBFS::WiiTDB::Covers::Disc:
-			mCache->cacheData( QWBFS::WiiTDB::Covers( url ).url( QWBFS::WiiTDB::Covers::DiscCustom ) );
+		case QWBFS::WiiTDB::CoverDisc: {
+			const QUrl enUrl = QWBFS::WiiTDB::coverUrl( QWBFS::WiiTDB::CoverDisc, id, "EN" );
+			
+			// request english
+			if ( url != enUrl ) {
+				mCache->get( QNetworkRequest( enUrl ) );
+			}
+			// or custom disk
+			else {
+				mCache->get( QNetworkRequest( QWBFS::WiiTDB::coverUrl( QWBFS::WiiTDB::CoverDiscCustom, id ) ) );
+			}
+			
 			return;
-		case QWBFS::WiiTDB::Covers::HQ:
-		case QWBFS::WiiTDB::Covers::Cover:
-		case QWBFS::WiiTDB::Covers::_3D:
-		case QWBFS::WiiTDB::Covers::DiscCustom:
-		case QWBFS::WiiTDB::Covers::Full:
-		case QWBFS::WiiTDB::Covers::Invalid:
+		}
+		case QWBFS::WiiTDB::CoverDiscCustom: {
+			const QUrl enUrl = QWBFS::WiiTDB::coverUrl( QWBFS::WiiTDB::CoverDiscCustom, id, "EN" );
+			
+			// request english
+			if ( url != enUrl ) {
+				mCache->get( QNetworkRequest( enUrl ) );
+				return;
+			}
+			
+			break;
+		}
+		case QWBFS::WiiTDB::Cover: {
+			const QUrl enUrl = QWBFS::WiiTDB::coverUrl( QWBFS::WiiTDB::Cover, id, "EN" );
+			
+			// request english
+			if ( url != enUrl ) {
+				mCache->get( QNetworkRequest( enUrl ) );
+				return;
+			}
+			
+			break;
+		}
+		case QWBFS::WiiTDB::CoverHQ:
+		case QWBFS::WiiTDB::Cover3D:
+		case QWBFS::WiiTDB::CoverFull:
+		case QWBFS::WiiTDB::CoverInvalid:
 			break;
 	}
 	
 	qmtbInfos->appendMessage( message );
 }
 
-void UIMain::dataNetworkCache_invalidated()
+void UIMain::networkAccessManager_cacheCleared()
 {
-	dataNetworkCache_dataCached( QUrl() );
+	networkAccessManager_cached( QUrl() );
 }
 
 void UIMain::on_aReloadPartitions_triggered()
 {
-	mPartitions.clear();
-
-#if defined( Q_OS_WIN )
-	foreach ( const QFileInfo& drive, QDir::drives() ) {
-		mPartitions << drive.absoluteFilePath().remove( ":" ).remove( "/" ).remove( "\\" );
-	}
-#elif defined( Q_OS_MAC )
-	QProcess process;
-	process.start( "diskutil list" );
-	process.waitForFinished();
+	PartitionComboBox::partitionModel()->update();
 	
-	const QStringList partitions = QString::fromLocal8Bit( process.readAll() ).split( "\n" );
-	
-	foreach ( QString partition, partitions ) {
-		partition = partition.trimmed();
-		
-		if ( partition.startsWith( "/" ) || partition.startsWith( "#" ) || partition.isEmpty() ) {
-			continue;
-		}
-		
-		partition = partition.simplified().section( ' ', -1 );
-		
-		// skip disks
-		if ( partition[ partition.size() -2 ].toLower() != 's' ) {
-			continue;
-		}
-		
-		mPartitions << QString( "/dev/%1" ).arg( partition );
-	}
-#elif defined( __linux__ )
-	QProcess process;
-	process.start( "cat /proc/partitions" );
-	process.waitForFinished();
-	
-	const QStringList partitions = QString::fromLocal8Bit( process.readAll() ).split( "\n" );
-	
-	foreach ( QString partition, partitions ) {
-		if ( partition.startsWith( "major" ) || partition.isEmpty() ) {
-			continue;
-		}
-		
-		partition = partition.simplified().section( ' ', -1 );
-		
-		// skip disks
-		if ( !partition[ partition.size() -1 ].isDigit() ) {
-			continue;
-		}
-		
-		mPartitions << QString( "/dev/%1" ).arg( partition );
-	}
-#else
-	QMessageBox::information( this, QString::null,
-		tr(
-			"I don't know how to list partition for this platform.\n"
-			"You will have to set the correct partition path yourself for mounting partitions."
-		) );
-#endif
-	
-	const QList<PartitionWidget*> widgets = sViews->findChildren<PartitionWidget*>();
-	
-	foreach ( PartitionWidget* widget, widgets ) {
-		widget->setPartitions( mPartitions );
+	if ( PartitionComboBox::partitionModel()->rowCount() == 0 ) {
+		qmtbInfos->appendMessage( tr(
+				"I don't know how to list partition for this platform.\n"
+				"You will have to set the correct partition path yourself for mounting partitions." ) );
 	}
 }
 
@@ -440,6 +476,95 @@ void UIMain::on_aProperties_triggered()
 	dlg->open();
 }
 
+void UIMain::on_aConvertToWBFSFiles_triggered()
+{
+	const QStringList filePaths = QFileDialog::getOpenFileNames( this, tr( "Choose ISO files to convert" ), QString::null, tr( "ISO Files (*.iso)" ) );
+	
+	if ( filePaths.isEmpty() ) {
+		return;
+	}
+	
+	ProgressDialog* dlg = new ProgressDialog( this );
+	
+	WorkerThread::Work work;
+	work.task = WorkerThread::ConvertWBFS;
+	
+	foreach ( const QString& filePath, filePaths ) {
+		work.discs << QWBFS::Model::Disc( filePath );
+	}
+	
+	work.target = QFileInfo( filePaths.first() ).absolutePath();
+	work.window = dlg;
+	
+	dlg->setWork( work );
+}
+
+void UIMain::on_aConvertToISOFiles_triggered()
+{
+	const QStringList filePaths = QFileDialog::getOpenFileNames( this, tr( "Choose WBFS files to convert" ), QString::null, tr( "WBFS Files (*.wbfs)" ) );
+	
+	if ( filePaths.isEmpty() ) {
+		return;
+	}
+	
+	ProgressDialog* dlg = new ProgressDialog( this );
+	
+	WorkerThread::Work work;
+	work.task = WorkerThread::ConvertISO;
+	
+	foreach ( const QString& filePath, filePaths ) {
+		work.discs << QWBFS::Model::Disc( filePath );
+	}
+	
+	work.target = QFileInfo( filePaths.first() ).absolutePath();
+	work.window = dlg;
+	
+	dlg->setWork( work );
+}
+
+void UIMain::on_aRenameDiscsInFolder_triggered()
+{
+	const QString path = QFileDialog::getExistingDirectory( this, tr( "Choose the folder to scan for ISOs/WBFSs files" ) );
+	
+	if ( path.isEmpty() ) {
+		return;
+	}
+	
+	/*
+		%title = Game Title
+		%id = Game ID
+		%suffix = File Suffix
+	*/
+	const QStringList patterns = QStringList()
+		<< "%id.%suffix" // GAMEID.wbfs
+		<< "%title [%id].%suffix" // Game Title [GAMEID].wbfs
+		<< "%id_%title/%id.%suffix" // GAMEID_Game Title/GAMEID.wbfs
+		<< "%title/%id.%suffix" // Game Title/GAMEID.wbfs
+		<< "%title[%id]/%id.%suffix" // Game Title[GAMEID]/GAMEID.wbfs
+		;
+	const QString text = tr( "Choose the pattern to apply:\n%1\n%2\n%3\n" )
+		.arg( tr( "%1 = Game Title" ).arg( "%title" ) )
+		.arg( tr( "%1 = Game Id" ).arg( "%id" ) )
+		.arg( tr( "%1 = File Suffix" ).arg( "%suffix" ) )
+		;
+	bool ok;
+	const QString pattern = QInputDialog::getItem( this, QString::null, text, patterns, 0, true, &ok );
+	
+	if ( !ok || pattern.isEmpty() ) {
+		return;
+	}
+	
+	ProgressDialog* dlg = new ProgressDialog( this );
+	
+	WorkerThread::Work work;
+	work.task = WorkerThread::RenameAll;
+	work.pattern = pattern;
+	work.target = path;
+	work.window = dlg;
+	
+	dlg->setWork( work );
+}
+
 void UIMain::on_tvFolders_activated( const QModelIndex& index )
 {
 	const QString filePath = mFoldersModel->filePath( index );
@@ -487,17 +612,17 @@ void UIMain::on_cbDrives_currentIndexChanged( const QString& text )
 
 void UIMain::on_tbClearExport_clicked()
 {
-	mExportModel->clear();
+	lvExport->model()->clear();
 }
 
 void UIMain::on_tbRemoveExport_clicked()
 {
-	mExportModel->removeSelection( lvExport->selectionModel()->selection() );
+	lvExport->model()->removeSelection( lvExport->selectionModel()->selection() );
 }
 
 void UIMain::on_tbExport_clicked()
 {
-	if ( mExportModel->rowCount() == 0 ) {
+	if ( lvExport->model()->rowCount() == 0 ) {
 		return;
 	}
 	
@@ -507,9 +632,39 @@ void UIMain::on_tbExport_clicked()
 		return;
 	}
 	
+	WorkerThread::Work work;
+	work.discs = lvExport->model()->discs();
+	work.target = path;
+	
+	const QMessageBox::StandardButtons buttons = QMessageBox::Yes | QMessageBox::Ok;
+	const QMessageBox::StandardButton defaultButton = QMessageBox::Ok;
+	
+	QMessageBox msgBox( this );
+	msgBox.setIcon( QMessageBox::Question );
+	msgBox.setText( tr( "Which format do you want to use to export the discs ?" ) );
+	msgBox.setStandardButtons( buttons );
+	msgBox.button( QMessageBox::Yes )->setText( "ISO" );
+	msgBox.button( QMessageBox::Ok )->setText( "WBFS" );
+	msgBox.setEscapeButton( 0 );
+	msgBox.setDefaultButton( defaultButton );
+	
+	switch ( msgBox.exec() ) {
+		case QMessageBox::Yes:
+			work.task = WorkerThread::ExportISO;
+			break;
+		case QMessageBox::Ok:
+			work.task = WorkerThread::ExportWBFS;
+			break;
+		default:
+			Q_ASSERT( 0 );
+			return;
+	}
+	
 	ProgressDialog* dlg = new ProgressDialog( this );
 	
 	connect( dlg, SIGNAL( jobFinished( const QWBFS::Model::Disc& ) ), this, SLOT( progress_jobFinished( const QWBFS::Model::Disc& ) ) );
 	
-	dlg->exportDiscs( mExportModel->discs(), path );
+	work.window = dlg;
+	
+	dlg->setWork( work );
 }
