@@ -42,10 +42,19 @@
 #include <ctime>
 
 #include <QDir>
+#include <QApplication>
 #include <QDebug>
 
+#if defined( Q_OS_WIN )
+#include <FreshCore/pWinHelpers>
+#include <qt_windows.h>
+#include <winioctl.h>
+#endif
+
 using namespace QWBFS;
 
+#define WBFS_FILE_MINIMUM_SIZE 1024 *1024 *16
+
 QMutex Driver::mMutex;
 bool Driver::mForce = false;
 QHash<QString, QWBFS::Partition::Handle> Driver::mHandles;
@@ -58,8 +67,28 @@ Driver::Driver( QObject* parent, const QWBFS::Partition::Handle& partitionHandle
 {
 	mProperties = partitionHandle.properties();
 	mHandle = partitionHandle;
-	mHasCreatedHandle = false;
+	mHasCreatedHandle = !partitionHandle.isValid();
 	
+	init();
+}
+
+Driver::Driver( const QWBFS::Partition::Handle& partitionHandle )
+	: QObject( 0 )
+{
+	mProperties = partitionHandle.properties();
+	mHandle = partitionHandle;
+	mHasCreatedHandle = !partitionHandle.isValid();
+	
+	init();
+}
+
+Driver::~Driver()
+{
+	close();
+}
+
+void Driver::init()
+{
 	if ( mRegions.isEmpty() && mLanguages.isEmpty() )
 	{
 		/*
@@ -77,57 +106,74 @@ Driver::Driver( QObject* parent, const QWBFS::Partition::Handle& partitionHandle
 			X 58 Not a real region code. Homebrew Channel uses it, though. 
 		*/
 		
-		// channel regions:
-		mRegions[ 'A' ] = QLatin1String( "World" );
-		mRegions[ 'D' ] = QLatin1String( "PAL" );
-		mRegions[ 'E' ] = QLatin1String( "NTSC-U" );
-		mRegions[ 'F' ] = QLatin1String( "PAL" );
-		mRegions[ 'J' ] = QLatin1String( "NTSC-J" );
-		mRegions[ 'K' ] = QLatin1String( "NTSC-K" );
-		mRegions[ 'L' ] = QLatin1String( "PAL-R" );
-		mRegions[ 'M' ] = QLatin1String( "PAL-R" );
-		mRegions[ 'N' ] = QLatin1String( "NTSC-T" );
-		mRegions[ 'P' ] = QLatin1String( "PAL" );
-		mRegions[ 'Q' ] = QLatin1String( "NTSC-T" );
-		mRegions[ 'T' ] = QLatin1String( "NTSC-T" );
-		// game regions:
-		mRegions[ 'H' ] = QLatin1String( "PAL" );
-		mRegions[ 'I' ] = QLatin1String( "PAL" );
-		mRegions[ 'S' ] = QLatin1String( "PAL" );
-		mRegions[ 'X' ] = QLatin1String( "PAL" );
-		mRegions[ 'Y' ] = QLatin1String( "PAL" );
-		mRegions[ 'W' ] = QLatin1String( "NTSC-J" );
-		mRegions[ 0 ] = QLatin1String( "NTSC-U" );
+		/*
+			WiiTDB Folder Game ID:
+			
+			E = US
+			J = JA
+			K = KO
+			R = RU
+			W = ZH
+			
+			ça c'est fixe, ça ne bouge pas, c'est toujours en fonction du game ID
+			
+			Pour les ID en P,D,F,I,S,H,X,Y,Z, on fait en fonction de la langue de l'utilisateur
+			(donc celle de l'interface ou avec une option) et on utilise EN, FR, DE, ES, IT, NL, PT
+			
+			Si l'image n'est pas trouvée dans la langue de préférence, on essaye EN (sauf si la langue préférée est l'anglais bien sur)
+			
+			
+			Si l'ID ne contient aucune de ces lettres c'est que c'est un jeu custom avec un ID qui n'est pas standard.
+			
+			Dans ce cas on essaye la langue préférée, puis EN, puis other en tout dernier.
+		*/
 		
-		// channel regions:
-		mLanguages[ 'A' ] = QLatin1String( "EN" ); // All regions. System channels like the Mii channel use it.
-		mLanguages[ 'D' ] = QLatin1String( "DE" ); // German-speaking regions.
-		mLanguages[ 'E' ] = QLatin1String( "US" ); // USA and other NTSC regions except Japan
-		mLanguages[ 'F' ] = QLatin1String( "FR" ); // French-speaking regions.
-		mLanguages[ 'J' ] = QLatin1String( "JA" ); // Japan
-		mLanguages[ 'K' ] = QLatin1String( "KO" ); // Korea
-		mLanguages[ 'L' ] = QLatin1String( "EN" ); // Japanese Import to Europe, Australia and other PAL regions
+		// Regions
+		mRegions.insert( 'A', "World" ); // All regions. System channels like the Mii channel use it.
+		mRegions.insert( 'E', "NTSC-U" ); // USA and other NTSC regions except Japan.
+		mRegions.insert( 'J', "NTSC-J" ); // Japan
+		mRegions.insert( 'K', "NTSC-K" ); // Korea
+		mRegions.insert( 'P', "PAL" ); // Europe, Australia and other PAL regions.
+		mRegions.insert( 'W', "NTSC-T" ); // Taiwan
+		mRegions.insert( 'R', "PAL-R" ); // Russia
+		mRegions.insert( 'Q', "NTSC-K" ); // Korea with Japanese language.
+		mRegions.insert( 'T', "NTSC-K" ); // Korea with English language.
+		mRegions.insert( 'F', "PAL" ); // French-speaking regions. Only if separate versions exist, e.g. Zelda: A Link to the Past.
+		mRegions.insert( 'D', "PAL" ); // German-speaking regions. Only if separate versions exist, e.g. Zelda: A Link to the Past.
+		mRegions.insert( 'I', "PAL" ); // Italien-speaking regions. Only if separate versions exist.
+		mRegions.insert( 'H', "PAL" ); // Dutch-speaking regions. Only if separate versions exist.
+		mRegions.insert( 'S', "PAL" ); // Spanish-speaking regions. Only if separate versions exist.
+		mRegions.insert( 'X', "PAL" ); // ??
+		mRegions.insert( 'Y', "PAL" ); // ??
+		mRegions.insert( 'Z', "PAL" ); // ??
+		// Custom disc / Channels
+		mRegions.insert( 'L', "CUSTOM" );
+		mRegions.insert( 'M', "CUSTOM" );
+		mRegions.insert( 'N', "CUSTOM" );
+		
+		// Languages
+		/*mLanguages[ 'L' ] = QLatin1String( "EN" ); // Japanese Import to Europe, Australia and other PAL regions
 		mLanguages[ 'M' ] = QLatin1String( "EN" ); // American Import to Europe, Australia and other PAL regions
-		mLanguages[ 'N' ] = QLatin1String( "JA" ); // Japanese Import to USA and other NTSC regions
-		mLanguages[ 'P' ] = QLatin1String( "EN" ); // Europe, Australia and other PAL regions
-		mLanguages[ 'Q' ] = QLatin1String( "KO" ); // Korea with Japanese language.
-		mLanguages[ 'T' ] = QLatin1String( "KO" ); // Korea with English language.
-		// game regions:
-		mLanguages[ 'H' ] = QLatin1String( "NL" ); // Netherlands
-		mLanguages[ 'I' ] = QLatin1String( "IT" ); // Italy
-		mLanguages[ 'S' ] = QLatin1String( "ES" ); // Spain
-		mLanguages[ 'X' ] = QLatin1String( "EN" );
-		mLanguages[ 'Y' ] = QLatin1String( "EN" );
-		mLanguages[ 'W' ] = QLatin1String( "ZH" );
-		mLanguages[ 0 ] = QLatin1String( "US" ); // Taiwan
+		mLanguages[ 'N' ] = QLatin1String( "JA" ); // Japanese Import to USA and other NTSC regions*/
+		
+		mLanguages.insert( 'A', "EN" ); // All regions. System channels like the Mii channel use it.
+		mLanguages.insert( 'E', "US" ); // USA and other NTSC regions except Japan.
+		mLanguages.insert( 'J', "JA" ); // Japan
+		mLanguages.insert( 'K', "KO" ); // Korea
+		mLanguages.insert( 'R', "RU" ); // Russia
+		mLanguages.insert( 'W', "ZH" ); // Taiwan
+		/*mLanguages.insert( 'Q', "JA" ); // Korea with Japanese language.
+		mLanguages.insert( 'T', "EN" ); // Korea with English language.
+		mLanguages.insert( 'F', "FR" ); // French-speaking regions. Only if separate versions exist, e.g. Zelda: A Link to the Past.
+		mLanguages.insert( 'D', "DE" ); // German-speaking regions. Only if separate versions exist, e.g. Zelda: A Link to the Past.
+		mLanguages.insert( 'I', "IT" ); // Italian-speaking regions. Only if separate versions exist.
+		mLanguages.insert( 'H', "NL" ); // Dutch-speaking regions. Only if separate versions exist.
+		mLanguages.insert( 'S', "ES" ); // Spanish-speaking regions. Only if separate versions exist.*/
+		
+		//insert( 'P', "" ); // Europe, Australia and other PAL regions.
 	}
 }
 
-Driver::~Driver()
-{
-	close();
-}
-
 void Driver::setPartition( const QString& partition )
 {
 	mProperties.partition = partition;
@@ -208,14 +254,16 @@ int Driver::discInfo( int index, QWBFS::Model::Disc& disc ) const
 	
 	const int headerSize = 0x100;
 	u8* header = (u8*)wbfs_ioalloc( headerSize );
+	quint32 size;
 	
 	disc.origin = mHandle.partition();
 
-	if ( wbfs_get_disc_info( mHandle.ptr(), index, header, headerSize, &disc.size ) != 0 ) {
+	if ( wbfs_get_disc_info( mHandle.ptr(), index, header, headerSize, &size ) != 0 ) {
 		wbfs_iofree( header );
 		return Driver::DiscReadFailed;
 	}
 	
+	disc.size = size *4ULL;
 	discInfo( header, disc );
 	wbfs_iofree( header );
 	
@@ -248,6 +296,10 @@ int Driver::usedBlocksCount() const
 
 int Driver::discImageInfo( const QString& fileName, QWBFS::Model::Disc& disc, partition_selector_t partitionSelection ) const
 {
+	if ( isWBFSPartitionOrFile( fileName ) ) {
+		return wbfsFileInfo( fileName, disc );
+	}
+	
 	if ( !isOpen() ) {
 		return Driver::PartitionNotOpened;
 	}
@@ -255,14 +307,15 @@ int Driver::discImageInfo( const QString& fileName, QWBFS::Model::Disc& disc, pa
 	void* fileHandle = wbfs_open_file_for_read( fileName.toLocal8Bit().data() );
 
 	if ( !fileHandle ) {
-		return Driver::DiscReadFailed;
+		return Driver::InvalidDisc;
 	}
 	
-	disc.origin = fileName;
 	u8* header = (u8*)wbfs_ioalloc( 0x100 );
-	disc.size = wbfs_estimate_disc( mHandle.ptr(), wbfs_read_wii_file, fileHandle, partitionSelection, header );
 	
+	disc.size = wbfs_estimate_disc( mHandle.ptr(), wbfs_read_wii_file, fileHandle, partitionSelection, header );
+	disc.origin = fileName;
 	discInfo( header, disc );
+	
 	wbfs_iofree( header );
 	wbfs_close_file( fileHandle );
 	
@@ -299,9 +352,9 @@ int Driver::addDiscImage( const QString& fileName, progress_callback_t progressC
 	
 	const u32 result2 = wbfs_add_disc( mHandle.ptr(), wbfs_read_wii_file, fileHandle, progressCallback, partitionSelection, copy1to1 ? 1 : 0
 #ifdef Q_OS_WIN
-	, newName.isEmpty() ? 0 : newName.toLocal8Bit().data()
+		, newName.isEmpty() ? 0 : newName.toLocal8Bit().data()
 #endif
-		);
+	);
 
 	wbfs_close_file( fileHandle );
 	return result2 == 0 ? Driver::Ok : Driver::DiscAddFailed;
@@ -337,21 +390,16 @@ int Driver::extractDisc( const QString& discId, const QString& path, const QStri
 	
 	const QString name = _name.isEmpty() ? discHandle.isoName() : _name;
 	const QString filePath = QDir::toNativeSeparators( QString( "%1/%2" ).arg( path ).arg( name ) );
-	void* fileHandle = wbfs_open_file_for_write( filePath.toLocal8Bit().data() );
+	int result = Driver::allocateFile( filePath, ( discHandle.ptr()->p->n_wii_sec_per_disc /2 ) *0x8000ULL );
+	void* fileHandle = result == Driver::Ok ? wbfs_open_file_for_write( filePath.toLocal8Bit().data() ) : 0;
 
 	if ( fileHandle ) {
-		// write a zero at the end of the iso to ensure the correct size
-		// XXX should check if the game is DVD9..
-		wbfs_file_reserve_space( fileHandle, ( discHandle.ptr()->p->n_wii_sec_per_disc /2 ) *0x8000ULL );
-		const int result = wbfs_extract_disc( discHandle.ptr(), wbfs_write_wii_file, fileHandle, progressCallback );
+		result = wbfs_extract_disc( discHandle.ptr(), wbfs_write_wii_file, fileHandle, progressCallback );
 		wbfs_close_file( fileHandle );
-		
-		if ( result == 0 ) {
-			return Driver::Ok;
-		}
-		else {
-			return Driver::DiscExtractFailed;
-		}
+		return result == 0 ? Driver::Ok : Driver::DiscExtractFailed;
+	}
+	else {
+		QFile::remove( filePath );
 	}
 	
 	return Driver::DiscWriteFailed;
@@ -423,7 +471,10 @@ int Driver::canDrive2Drive( const QWBFS::Partition::Handle& sourcePartitionHandl
 		return Driver::SourcePartitionNotOpened;
 	}
 	
-	if ( ( sourcePartitionHandle.ptr()->wbfs_sec_sz /sourcePartitionHandle.ptr()->hd_sec_sz ) == ( mHandle.ptr()->wbfs_sec_sz /mHandle.ptr()->hd_sec_sz ) ) {
+	const double left = sourcePartitionHandle.ptr()->wbfs_sec_sz /sourcePartitionHandle.ptr()->hd_sec_sz;
+	const double right = mHandle.ptr()->wbfs_sec_sz /mHandle.ptr()->hd_sec_sz;
+	
+	if ( qFuzzyCompare( left, right ) ) {
 		return Driver::Ok;
 	}
 	
@@ -467,13 +518,49 @@ int Driver::discList( QWBFS::Model::DiscList& list ) const
 	return Driver::Ok;
 }
 
-bool Driver::isWBFSPartition( const QString& fileName )
+int Driver::trim() const
 {
-	if ( !QFile::exists( fileName ) ) {
-		return false;
+	if ( !isOpen() ) {
+		return Driver::PartitionNotOpened;
 	}
 	
-	QFile file( fileName );
+	wbfs_trim( mHandle.ptr() );
+	wbfs_file_truncate( mHandle.ptr()->callback_data, mHandle.ptr()->n_hd_sec *512ULL );
+	
+	return Driver::Ok;
+}
+
+bool Driver::isWBFSPartitionOrFile( const QString& _fileName )
+{
+	QString filePath = _fileName;
+	
+#if defined( Q_OS_WIN )
+	if ( !filePath.isEmpty() && filePath.length() <= 3 ) {
+		filePath = QString( "\\\\?\\%1:" ).arg( filePath[ 0 ] );
+		DISK_GEOMETRY diskGeometry;
+		HANDLE handle = CreateFile( QStringToTCHAR( filePath ), GENERIC_READ | FILE_SHARE_READ, 0, NULL, OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, NULL );
+		
+		if ( handle != INVALID_HANDLE_VALUE ) {
+			DWORD bytes;
+			
+			if ( DeviceIoControl( handle, IOCTL_DISK_GET_DRIVE_GEOMETRY, NULL, 0, &diskGeometry, sizeof(DISK_GEOMETRY), &bytes, NULL ) ) {
+				const DWORD sectorSize = diskGeometry.BytesPerSector;
+				char buffer[ sectorSize  ];
+				DWORD read;
+				
+				if ( ReadFile( handle, buffer, sectorSize, &read, NULL ) ) {
+					CloseHandle( handle );
+					return QByteArray( buffer ).left( 4 ).toLower() == "wbfs";
+				}
+			}
+		}
+		
+		CloseHandle( handle );
+		return false;
+	}
+#endif
+	
+	QFile file( filePath );
 	
 	if ( file.open( QIODevice::ReadOnly ) ) {
 		return file.read( 4 ).toLower() == "wbfs";
@@ -482,35 +569,156 @@ bool Driver::isWBFSPartition( const QString& fileName )
 	return false;
 }
 
+int Driver::allocateFile( const QString& filePath, qint64 size )
+{
+	const int go = 1024 *1024 *1024;
+	QFile file( filePath );
+	
+	if ( file.exists() ) {
+		return Driver::DiscFound;
+	}
+	
+	if ( file.open( QIODevice::WriteOnly ) ) {
+		if ( size == -1 ) {
+			if ( file.resize( go *8.8 ) ) { // 8.8 go - all FS except FAT32 // 143432 *2 *0x8000ULL
+				return Driver::Ok;
+			}
+			else if ( file.resize( ( go *4.0 ) -1 ) ) { // 4go - fat32 FS
+				return Driver::Ok;
+			}
+		}
+		else if ( file.resize( size ) ) {
+			return Driver::Ok;
+		}
+	}
+	
+	file.remove();
+	
+	return Driver::DiscWriteFailed;
+}
+
+int Driver::wbfsFileInfo( const QString& wbfsFileName, QWBFS::Model::Disc& disc )
+{
+	/*if ( !isWBFSPartitionOrFile( wbfsFileName ) ) {
+		return Driver::InvalidDisc;
+	}*/
+	
+	wbfs_t* partition = wbfs_try_open_partition( wbfsFileName.toLocal8Bit().data(), 0 );
+	
+	if ( !partition ) {
+		return Driver::InvalidDisc;
+	}
+	
+	u8* header = (u8*)wbfs_ioalloc( 0x100 );
+	int count = wbfs_count_discs( partition );
+	
+	if ( count != 1 ) {
+		wbfs_iofree( header );
+		wbfs_close( partition );
+		return Driver::InvalidDisc;
+	}
+	
+	wbfs_disc_t* cd;
+	quint32 size;
+	
+	if( !wbfs_get_disc_info( partition, 0, header, 0x100, &size ) ) {
+		cd = wbfs_open_disc( partition, header );
+		
+		if ( !cd ) {
+			wbfs_iofree( header );
+			wbfs_close( partition );
+			return Driver::InvalidDisc;
+		}
+		
+		wbfs_close_disc( cd );
+	}
+	
+	disc.size = size *4ULL;
+	disc.origin = wbfsFileName;
+	discInfo( header, disc );
+	
+	wbfs_iofree( header );
+	wbfs_close( partition );
+	
+	return disc.size != 0 ? Driver::Ok : Driver::InvalidDisc;
+}
+
+int Driver::convertIsoFileToWBFSFile( const QString& isoFilePath, const QString& _wbfsFilePath )
+{
+	int result;
+	QString wbfsFilePath = _wbfsFilePath;
+	
+	if ( wbfsFilePath.isEmpty() ) {
+		wbfsFilePath = QString( "%1.wbfs" ).arg( isoFilePath );
+	}
+	
+	result = Driver::allocateFile( wbfsFilePath );
+	
+	if ( result != QWBFS::Driver::Ok ) {
+		return result;
+	}
+	
+	QWBFS::Partition::Properties properties( wbfsFilePath );
+	properties.reset = true;
+	
+	QWBFS::Partition::Handle handle( properties );
+	QWBFS::Driver td( 0, handle );
+	
+	if ( !td.isOpen() ) {
+		return Driver::PartitionNotOpened;
+	}
+	
+	QWBFS::Model::Disc disc;
+	result = td.discImageInfo( isoFilePath, disc );
+	
+	if ( result != QWBFS::Driver::Ok ) {
+		return result;
+	}
+	
+	result = td.addDiscImage( disc.origin );
+	
+	if ( result != QWBFS::Driver::Ok ) {
+		return result;
+	}
+	
+	return td.trim();
+}
+
 QString Driver::errorToString( QWBFS::Driver::Error error )
 {
 	switch ( error ) {
 		case Driver::Ok:
-			return tr( "No error." );
+			return tr( "No error" );
 		case Driver::PartitionNotOpened:
-			return tr( "Partition not opened." );
+			return tr( "Partition not opened" );
 		case Driver::SourcePartitionNotOpened:
-			return tr( "Source partition not opened." );
+			return tr( "Source partition not opened" );
 		case Driver::DiscReadFailed:
-			return tr( "Disc read failed." );
+			return tr( "Disc read failed" );
 		case Driver::DiscWriteFailed:
-			return tr( "Disc write failed." );
+			return tr( "Disc write failed" );
 		case Driver::DiscExtractFailed:
-			return tr( "Disc extract failed." );
+			return tr( "Disc extract failed" );
 		case Driver::DiscAddFailed:
-			return tr( "Disc add failed." );
+			return tr( "Disc add failed" );
+		case Driver::DiscConvertFailed:
+			return tr( "Disc convert failed" );
+		case Driver::DiscRenameFailed:
+			return tr( "Disc rename failed" );
 		case Driver::DiscFound:
-			return tr( "Disc found." );
+			return tr( "Disc found" );
 		case Driver::DiscNotFound:
-			return tr( "Disc not found (or not exists)." );
+			return tr( "Disc not found (or not exists)" );
 		case Driver::InvalidDiscIndex:
-			return tr( "Invalid disc index." );
+			return tr( "Invalid disc index" );
 		case Driver::InvalidDiscID:
-			return tr( "Invalid disc id." );
+			return tr( "Invalid disc id" );
 		case Driver::InvalidDisc:
-			return tr( "Invalid disc." );
+			return tr( "Invalid disc" );
 		case Driver::CantDrive2Drive:
-			return tr( "Can't drive to drive copy." );
+			return tr( "Can't drive to drive copy" );
+		case Driver::UnknownError:
+			return tr( "Unknown error" );
 	}
 	
 	return QString::null;
@@ -518,12 +726,13 @@ QString Driver::errorToString( QWBFS::Driver::Error error )
 
 QString Driver::regionToString( int region )
 {
-	return mRegions.value( region, mRegions.value( 0 ) );
+	return mRegions.value( region, "Unknow Region" );
 }
 
 QString Driver::regionToLanguageString( int region )
 {
-	return mLanguages.value( region, mLanguages.value( 0 ) );
+	const QString systemLanguage = QLocale::system().name().section( '_', 0, 0 ).toUpper();
+	return mLanguages.value( region, systemLanguage.isEmpty() ? "EN" : systemLanguage );
 }
 
 QString Driver::stateToString( QWBFS::Driver::State state )
@@ -616,6 +825,80 @@ void Driver::discInfo( u8* header, QWBFS::Model::Disc& disc )
 	disc.region = QChar( header[ 0x3 ] ).unicode();
 }
 
+qint64 Driver::minimumWBFSFileSize()
+{
+	return WBFS_FILE_MINIMUM_SIZE;
+}
+
+QWBFS::Model::Disc Driver::isoDiscInfo( const QString& filePath )
+{
+	const QString tmpFile = QString( "%1/%2.tmp%3" )
+		.arg( QDir::tempPath() )
+		.arg( QApplication::applicationName() )
+		.arg( minimumWBFSFileSize() );
+	QWBFS::Model::Disc disc;
+	
+	// create temporary wbfs container for estimating the size
+	if ( !QFile::exists( tmpFile ) ) {
+		if ( Driver::allocateFile( tmpFile, minimumWBFSFileSize() ) != QWBFS::Driver::Ok ) {
+			qWarning() << Q_FUNC_INFO << "Can't create tmp file" << tmpFile.toLocal8Bit().constData();
+			return disc;
+		}
+	}
+	
+	QWBFS::Partition::Properties properties( tmpFile );
+	properties.reset = true;
+	
+	QWBFS::Partition::Handle handle( properties );
+	
+	if ( handle.isValid() ) {
+		void* fileHandle = wbfs_open_file_for_read( filePath.toLocal8Bit().data() );
+
+		if ( fileHandle ) {
+			u8* header = (u8*)wbfs_ioalloc( 0x100 );
+			
+			disc.size = wbfs_estimate_disc( handle.ptr(), wbfs_read_wii_file, fileHandle, ONLY_GAME_PARTITION, header );
+			disc.origin = filePath;
+			discInfo( header, disc );
+			
+			wbfs_iofree( header );
+			wbfs_close_file( fileHandle );
+		}
+		else {
+			qWarning() << Q_FUNC_INFO << "Invalid file handle";
+		}
+	}
+	else {
+		qWarning() << Q_FUNC_INFO << "Invalid wbfs handle";
+	}
+	
+	//QFile::remove( tmpFile ); // should delete it but may be not performant for multiple requests of iso estimated size
+	
+	return disc;
+}
+
+Driver::FileType Driver::fileType( const QString& filePath )
+{
+	const QFileInfo file( filePath );
+	const QString suffix = file.isDir() ? QString::null : file.suffix().toLower();
+	
+	if ( suffix == "iso" ) {
+		return QWBFS::Driver::ISOFile;
+	}
+	else if ( suffix == "wbfs" ) {
+		if ( isWBFSPartitionOrFile( filePath ) ) {
+			return QWBFS::Driver::WBFSFile;
+		}
+	}
+	else if ( suffix.isEmpty() ) {
+		if ( isWBFSPartitionOrFile( filePath ) ) {
+			return QWBFS::Driver::WBFSPartitionFile;
+		}
+	}
+	
+	return QWBFS::Driver::UnknownFile;
+}
+
 int Driver::discRead_callback( void* fp, u32 lba, u32 count, void* iobuf )
 {
 	int ret = wbfs_disc_read( (wbfs_disc_t*)fp, lba, (u8*)iobuf, count );
@@ -638,7 +921,7 @@ int Driver::discRead_callback( void* fp, u32 lba, u32 count, void* iobuf )
 	return 0;
 }
 
-void Driver::progress_callback( int x, int max )
+QTime Driver::estimatedTimeForTask( int x, int max )
 {
 	static time_t start_time;
 	static u32 expected_total;
@@ -656,11 +939,7 @@ void Driver::progress_callback( int x, int max )
 		m = ( d /60 ) %60;
 		s = d %60;
 		
-		if ( mCurrentDriver ) {
-			emit mCurrentDriver->currentProgressChanged( x, max, QTime( h, m, s ) );
-		}
-		
-		return;
+		return QTime( h, m, s );
 	}
 
 	d = (u32)( time( 0 ) -start_time );
@@ -680,7 +959,12 @@ void Driver::progress_callback( int x, int max )
 	m = ( d /60 ) %60;
 	s = d %60;
 	
+	return QTime( h, m, s );
+}
+
+void Driver::progress_callback( int x, int max )
+{
 	if ( mCurrentDriver ) {
-		emit mCurrentDriver->currentProgressChanged( x, max, QTime( h, m, s ) );
+		emit mCurrentDriver->currentProgressChanged( x, max, estimatedTimeForTask( x, max ) );
 	}
 }
